Package: MsBackendMgf
Title: Mass Spectrometry Data Backend for Mascot Generic Format (mgf)
        Files
Version: 1.19.0
Authors@R: 
    c(person(given = "RforMassSpectrometry Package Maintainer",
             email = "maintainer@rformassspectrometry.org",
             role = "cre"),
      person(given = "Laurent", family = "Gatto",
             role = "aut",
             email = "laurent.gatto@uclouvain.be",
             comment = c(ORCID = "0000-0002-1520-2268")),
      person(given = "Johannes", family = "Rainer",
             email = "Johannes.Rainer@eurac.edu",
             role = "aut",
             comment = c(ORCID = "0000-0002-6977-7147")),
      person(given = "Sebastian", family = "Gibb",
             comment = c(ORCID = "0000-0001-7406-4443"),
             email = "mail@sebastiangibb.de",
             role = "aut"),
      person(given = "Michael", family = "Witting",
             comment = c(ORCID = "0000-0002-1462-4426"),
             email = "michael.witting@helmholtz-muenchen.de",
             role = "ctb"),
      person(given = "Adriano", family = "Rutz",
             comment = c(ORCID = "0000-0003-0443-9902"),
	     email = "rutz@imsb.biol.ethz.ch",
	     role = "ctb"),
	     person(given = "Corey", family = "Broeckling",
	     comment = c(ORCID = "0000-0002-6158-827X"),
	     email = "Corey.Broeckling@ColoState.edu", role = "ctb"))
Description: Mass spectrometry (MS) data backend supporting import and
	     export of MS/MS spectra data from Mascot Generic Format
	     (mgf) files. Objects defined in this package are supposed to be
	     used with the Spectra Bioconductor package. This package thus
	     adds mgf file support to the Spectra package.
Depends: R (>= 4.0), Spectra (>= 1.5.14)
Imports: ProtGenerics (>= 1.35.3), BiocParallel, S4Vectors, IRanges,
        MsCoreUtils, methods, stats
Suggests: testthat, knitr (>= 1.1.0), roxygen2, BiocStyle (>= 2.5.19),
        rmarkdown
License: Artistic-2.0
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/RforMassSpectrometry/MsBackendMgf/issues
URL: https://github.com/RforMassSpectrometry/MsBackendMgf
biocViews: Infrastructure, Proteomics, MassSpectrometry, Metabolomics,
        DataImport
Roxygen: list(markdown=TRUE)
RoxygenNote: 7.3.2
Collate: 'hidden_aliases.R' 'MsBackendMgf.R' 'functions-mgf.R'
git_url: https://git.bioconductor.org/packages/MsBackendMgf
git_branch: devel
git_last_commit: 3de3a3e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 23:00:21 UTC; biocbuild
Author: RforMassSpectrometry Package Maintainer [cre],
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>),
  Johannes Rainer [aut] (ORCID: <https://orcid.org/0000-0002-6977-7147>),
  Sebastian Gibb [aut] (ORCID: <https://orcid.org/0000-0001-7406-4443>),
  Michael Witting [ctb] (ORCID: <https://orcid.org/0000-0002-1462-4426>),
  Adriano Rutz [ctb] (ORCID: <https://orcid.org/0000-0003-0443-9902>),
  Corey Broeckling [ctb] (ORCID: <https://orcid.org/0000-0002-6158-827X>)
Maintainer: RforMassSpectrometry Package Maintainer <maintainer@rformassspectrometry.org>
