# Generated by roxygen2: do not edit by hand

export(MsBackendAnnotatedMgf)
export(MsBackendMgf)
export(readMgf)
export(readMgfSplit)
exportClasses(MsBackendMgf)
exportMethods(backendInitialize)
exportMethods(export)
exportMethods(spectraVariableMapping)
importClassesFrom(Spectra,MsBackendDataFrame)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpmapply)
importFrom(IRanges,NumericList)
importFrom(MsCoreUtils,rbindFill)
importFrom(S4Vectors,DataFrame)
importFrom(Spectra,coreSpectraVariables)
importFrom(methods,as)
importFrom(methods,new)
importFrom(methods,validObject)
importFrom(stats,setNames)
importMethodsFrom(BiocParallel,bplapply)
importMethodsFrom(ProtGenerics,backendInitialize)
importMethodsFrom(ProtGenerics,peaksData)
importMethodsFrom(S4Vectors,metadata)
importMethodsFrom(Spectra,"$")
importMethodsFrom(Spectra,"$<-")
importMethodsFrom(Spectra,"spectraData<-")
importMethodsFrom(Spectra,backendInitialize)
importMethodsFrom(Spectra,export)
importMethodsFrom(Spectra,spectraData)
importMethodsFrom(Spectra,spectraNames)
importMethodsFrom(Spectra,spectraVariableMapping)
importMethodsFrom(Spectra,spectraVariables)
